@@ -207,9 +207,7 @@ Gnuplot can plot both data and equations. This module exists largely for the
 data-plotting case, but sometimes it can be useful to plot equations together
 with some data. This is supported by the 'equation' plot option. This plot
 option is either a string (for a single equation) or a list/tuple containing
-multiple strings for multiple equations. Note that plotting only equations
-without data is not supported (and generally is better done with gnuplot
-directly). An example:
+multiple strings for multiple equations. An example:
 
     import numpy as np
     import numpy.random as nr
@@ -371,14 +369,18 @@ equation) or an array/tuple of strings (for multiple equations). See the
 
 Instead of drawing a plot on screen, plot into a file instead. The output
 filename is the value associated with this key. The output format is inferred
-from the filename. Currently only eps, ps, pdf, png, svg are supported with some
-default sets of options. This option is simply a shorthand for the 'terminal'
-and 'output' options. If the defaults provided by the 'hardcopy' option are
-insufficient, use 'terminal' and 'output' manually. Example:
+from the filename. Currently only eps, ps, pdf, png, svg, gp are supported with
+some default sets of options. This option is simply a shorthand for the
+'terminal' and 'output' options. If the defaults provided by the 'hardcopy'
+option are insufficient, use 'terminal' and 'output' manually. Example:
 
     plot(..., hardcopy="plot.pdf")
     [ Plots into that file ]
 
+Note that the ".gp" format is special. Instead of asking gnuplot to make a plot
+using a specific terminal, writing to "xxx.gp" will create a self-plotting data
+file that is visualized with gnuplot.
+
 - terminal
 
 Selects the gnuplot terminal (backend). This determines how Gnuplot generates
@@ -505,6 +507,20 @@ Specifies how many values represent each data point. For 2D plots this defaults
 to 2; for 3D plots this defaults to 3. These defaults are correct for simple
 plots
 
+- using
+
+Overrides the 'using' directive we pass to gnuplot. No error checking is
+performed, and the string is passed to gnuplot verbatim. This option is very
+rarely needed. The most common usage is to apply a function to an implicit
+domain. For instance, this basic command plots a line (linearly increasing
+values) against a linearly-increasing line number::
+
+    gp.plot(np.arange(100))
+
+We can plot the same values against the square-root of the line number to get a
+parabola:
+
+    gp.plot(np.arange(100), using='(sqrt($1)):2')
 
 * INTERFACE
 
@@ -686,10 +702,10 @@ To send any plot to a file, instead of to the screen, one can simply do
     plot(x, y,
          hardcopy = 'output.pdf')
 
-The 'hardcopy' option is a shorthand for the 'terminal' and 'output'
-options. If more control is desired, the latter can be used. For example to
-generate a PDF of a particular size with a particular font size for the text,
-one can do
+The 'hardcopy' option is a shorthand for the 'terminal' and 'output' options (in
+all cases except when writing a .gp file; see below). If more control is
+desired, the latter can be used. For example to generate a PDF of a particular
+size with a particular font size for the text, one can do
 
     plot(x, y,
          terminal = 'pdfcairo solid color font ",10" size 11in,8.5in',
@@ -698,6 +714,14 @@ one can do
 This command is equivalent to the 'hardcopy' shorthand used previously, but the
 fonts and sizes can be changed.
 
+If we write to a ".gp" file:
+
+    plot(x, y,
+         hardcopy = 'data.gp')
+
+then instead of running gnuplot, we create a self-plotting file. gnuplot is
+invoked when we execute that file.
+
 '''
 
 
@@ -715,10 +739,10 @@ import numpy as np
 import numpysane as nps
 
 # setup.py assumes the version is a simple string in '' quotes
-__version__ = '0.20'
+__version__ = '0.23'
 
 # note that 'with' is both a known plot and curve option
-knownPlotOptions = frozenset(('dump', 'ascii', 'log', 'notest',
+knownPlotOptions = frozenset(('dump', 'ascii', 'log', 'notest', 'wait',
 
                               '3d',
                               'cmds', 'set', 'unset', 'square', 'square_xy', 'title',
@@ -730,7 +754,7 @@ knownPlotOptions = frozenset(('dump', 'ascii', 'log', 'notest',
                               'zmax',  'zmin',  'zrange',  'zinv',  'zlabel',
                               'cbmin', 'cbmax', 'cbrange'))
 
-knownCurveOptions = frozenset(('legend', 'y2', 'with', 'tuplesize'))
+knownCurveOptions = frozenset(('legend', 'y2', 'with', 'tuplesize', 'using'))
 
 knownInteractiveTerminals = frozenset(('x11', 'wxt', 'qt', 'aquaterm'))
 
@@ -747,10 +771,20 @@ def _getGnuplotFeatures():
     # DISPLAY. I'm not actually plotting anything, so a DISPLAY can try to
     # X-forward and be really slow pointlessly
 
+    # I pass in the current environment, but with DISPLAY turned off
+    env = os.environ.copy()
+    env['DISPLAY'] = ''
+
     # first, I run 'gnuplot --help' to extract all the cmdline options as features
-    helpstring = subprocess.check_output(['gnuplot', '--help'],
-                                         stderr=subprocess.STDOUT,
-                                         env={'DISPLAY': ''})
+    try:
+        helpstring = subprocess.check_output(['gnuplot', '--help'],
+                                             stderr=subprocess.STDOUT,
+                                             env=env).decode()
+    except FileNotFoundError:
+        print("Couldn't run gnuplot. Is it installed? Is it findable in the PATH?",
+              file=sys.stderr)
+        raise
+
     features = set( re.findall(r'--([a-zA-Z0-9_]+)', helpstring) )
 
 
@@ -758,7 +792,7 @@ def _getGnuplotFeatures():
     try:
         out = subprocess.check_output(('gnuplot', '-e', "set view equal"),
                                       stderr=subprocess.STDOUT,
-                                      env={'DISPLAY': ''})
+                                      env=env).decode()
     except:
         out = 'error!'
 
@@ -797,6 +831,12 @@ class GnuplotlibError(Exception):
 
 
 
+def _data_dump_only(plotOptions):
+    '''Returns True if we're dumping a script, NOT actually running gnuplot'''
+    return \
+        plotOptions.get('dump') or \
+        plotOptions.get('terminal') == 'gp' or \
+        re.match(".*\.gp$",plotOptions.get('hardcopy',''))
 
 
 class gnuplotlib:
@@ -805,11 +845,16 @@ class gnuplotlib:
 
         # some defaults
         self.plotOptions      = _dictDeUnderscore(plotOptions)
+        self._dumpPipe        = None
         self.t0               = time.time()
         self.checkpoint_stuck = False
         self.sync_count       = 1
 
-        plotOptionsCmds = self._getPlotOptionsCmds()
+        self._plotOptionsCmds = self._getPlotOptionsCmds()
+
+        if _data_dump_only(self.plotOptions):
+            self.gnuplotProcess = None
+            return
 
         # if we already have a gnuplot process, reset it. Otherwise make a new
         # one
@@ -821,23 +866,41 @@ class gnuplotlib:
             self._startgnuplot()
             self._logEvent("_startgnuplot() finished")
 
-        self._safelyWriteToPipe(plotOptionsCmds)
+        self._safelyWriteToPipe(self._plotOptionsCmds)
 
 
     def _startgnuplot(self):
 
         self._logEvent("_startgnuplot()")
 
-        if not self.plotOptions.get('dump'):
+        cmd = ['gnuplot']
 
-            cmd = ['gnuplot']
+        try:
+            self.fdDupSTDOUT = os.dup(sys.stdout.fileno())
+        except:
+            self.fdDupSTDOUT = None
 
-            try:
-                self.fdDupSTDOUT = os.dup(sys.stdout.fileno())
-            except:
-                self.fdDupSTDOUT = None
+        self.gnuplotProcess = \
+            subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
 
-            self.gnuplotProcess = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+                             # required to "autoflush" writes
+                             bufsize=0,
+
+                             # This was helpful in python3 to implicitly
+                             # encode() strings, but it broke the
+                             # select()/read() mechanism: select() would
+                             # look at the OS file descriptor, but read()
+                             # would look at some buffer, so you'd get into
+                             # a situation where
+                             #
+                             # - data was read from the OS into a buffer, and is available to be read()
+                             # - select() blocks waiting for MORE data
+                             #
+                             # I guess I leave this off and manully
+                             # encode/decode everything
+
+                             #encoding = 'utf-8',
+            )
 
         # save the default terminal
         self._safelyWriteToPipe("set terminal push", 'terminal')
@@ -976,9 +1039,9 @@ defaults are acceptable, use 'hardcopy' only, otherwise use 'terminal' and
 'output' to get more control""")
 
             outputfile = self.plotOptions['hardcopy']
-            m = re.search(r'\.(eps|ps|pdf|png|svg)$', outputfile)
+            m = re.search(r'\.(eps|ps|pdf|png|svg|gp)$', outputfile)
             if not m:
-                raise GnuplotlibError("Only .eps, .ps, .pdf, .png and .svg hardcopy output supported")
+                raise GnuplotlibError("Only .eps, .ps, .pdf, .png, .svg and .gp hardcopy output supported")
 
             outputfileType = m.group(1)
 
@@ -986,18 +1049,22 @@ defaults are acceptable, use 'hardcopy' only, otherwise use 'terminal' and
                              'ps':  'postscript noenhanced solid color landscape 10',
                              'pdf': 'pdf noenhanced solid color font ",10" size 11in,8.5in',
                              'png': 'png noenhanced size 1280,1024',
-                             'svg': 'svg noenhanced solid'}
+                             'svg': 'svg noenhanced solid',
+                             'gp':  'gp'}
 
             self.plotOptions['terminal'] = terminalOpts[outputfileType]
             self.plotOptions['output']   = outputfile
 
-
         if 'terminal' in self.plotOptions:
             if self.plotOptions['terminal'] in knownInteractiveTerminals:
                 # known interactive terminal
                 if 'output' in self.plotOptions and self.plotOptions['output'] != '':
                     sys.stderr.write("Warning: requested a known-interactive gnuplot terminal AND an output file. Is this REALLY what you want?\n")
 
+            if self.plotOptions['terminal'] == 'gp':
+                self.plotOptions['dump'  ] = 1
+                self.plotOptions['notest'] = 1
+
         # add the extra global options
         if 'cmds' in self.plotOptions:
             # if there's a single cmds option, put it into a 1-element list to
@@ -1018,7 +1085,10 @@ defaults are acceptable, use 'hardcopy' only, otherwise use 'terminal' and
             else:
                 self._printGnuplotPipe( "exit\n" )
 
-            self.gnuplotProcess.wait()
+            try:
+                self.gnuplotProcess.wait()
+            except:
+                pass
             self.gnuplotProcess = None
 
             if self.fdDupSTDOUT is not None:
@@ -1084,10 +1154,30 @@ defaults are acceptable, use 'hardcopy' only, otherwise use 'terminal' and
     def _gnuplotStdin(self):
         if self.gnuplotProcess:
             return self.gnuplotProcess.stdin
-        return sys.stdout
+
+        # In python2 I just return stdout. But the python3 people have no idea
+        # what they're doing. The normal pipe return by Popen is a FileIO, so I
+        # can ONLY write bytes to it; if I write a string to it, it barfs. So I
+        # normally need to do the encode/decode dance. But sys.stdout is a
+        # TextIOWrapper, which means that I must write STRINGS and it'll barf if
+        # I write bytes. I can apparently reach inside and grab the
+        # corresponding FileIO object to make it work like the pipe, so I do
+        # that
+        # debug dump. I return stdout
+        if self._dumpPipe:
+            try:
+                return self._dumpPipe.buffer.raw
+            except:
+                return self._dumpPipe
+
+        try:
+            return sys.stdout.buffer.raw
+        except:
+            return sys.stdout
 
     def _printGnuplotPipe(self, string):
-        self._gnuplotStdin().write( string )
+
+        self._gnuplotStdin().write( string.encode() )
         self._logEvent("Sent to child process {} bytes ==========\n{}=========================".
                        format(len(string), string))
 
@@ -1137,7 +1227,7 @@ defaults are acceptable, use 'hardcopy' only, otherwise use 'terminal' and
         # yet arrived. I thus print out a checkpoint message and keep reading the
         # child's STDERR pipe until I get that message back. Any errors would have
         # been printed before this
-        if self.plotOptions.get('notest'):
+        if self.plotOptions.get('notest') and flags != 'waitforever':
             return None, None
 
         checkpoint = "gpsync{}xxx".format(self.sync_count)
@@ -1159,7 +1249,8 @@ defaults are acceptable, use 'hardcopy' only, otherwise use 'terminal' and
 
             self._logEvent("Trying to read from gnuplot")
 
-            rlist,wlist,xlist = select.select([self.gnuplotProcess.stderr],[], [], 15)
+            rlist,wlist,xlist = select.select([self.gnuplotProcess.stderr],[], [],
+                                              None if flags == 'waitforever' else 15)
 
             if rlist:
                 # read a byte. I'd like to read "as many bytes as are
@@ -1168,7 +1259,7 @@ defaults are acceptable, use 'hardcopy' only, otherwise use 'terminal' and
                 # simply do a non-blocking read). Very little data will be
                 # coming in anyway, so doing this a byte at a time is an
                 # irrelevant inefficiency
-                byte = self.gnuplotProcess.stderr.read(1)
+                byte = self.gnuplotProcess.stderr.read(1).decode()
                 fromerr += byte
                 self._logEvent("Read byte '{}' ({}) from gnuplot child process".format(byte, hex(ord(byte))))
             else:
@@ -1265,10 +1356,10 @@ and/or gnuplot itself. Please report this as a gnuplotlib bug''')
         pipe = self._gnuplotStdin()
         if self.plotOptions.get('ascii'):
             if curve.get('matrix'):
-                np.savetxt( pipe,
-                            nps.glue(*curve['_data'], axis=-2).astype(np.float64,copy=False),
-                            '%s' )
-                pipe.write("\ne\n")
+                np.savetxt(pipe,
+                           nps.glue(*curve['_data'], axis=-2).astype(np.float64,copy=False),
+                           '%s')
+                pipe.write(b"\ne\n")
             else:
                 # Previously I was doing this:
                 #     np.savetxt( pipe,
@@ -1288,20 +1379,22 @@ and/or gnuplot itself. Please report this as a gnuplotlib bug''')
 labels with spaces in them
 
                     '''
-                    if type(e) is np.string_:
-                        pipe.write('"')
-                        pipe.write(str(e))
-                        pipe.write('"')
+                    if type(e) is np.string_ or type(e) is np.str_:
+                        pipe.write(b'"')
+                        pipe.write(str(e).encode())
+                        pipe.write(b'"')
                     else:
-                        pipe.write(str(e))
+                        pipe.write(str(e).encode())
 
-                for i in xrange(curve['_data'][0].shape[-1]):
-                    for j in xrange(Ncurves-1):
+                for i in range(curve['_data'][0].shape[-1]):
+                    for j in range(Ncurves-1):
                         write_element(curve['_data'][j][i])
-                        pipe.write(' ')
+                        pipe.write(b' ')
                     write_element(curve['_data'][Ncurves-1][i])
-                    pipe.write('\n')
-                pipe.write("e\n")
+                    pipe.write(b'\n')
+
+
+                pipe.write(b"e\n")
 
         else:
             nps.mv(nps.cat(*curve['_data']), 0, -1).astype(np.float64,copy=False).tofile(pipe)
@@ -1348,7 +1441,10 @@ labels with spaces in them
             if curve.get('matrix'):
                 using_Ncolumns -= 2
 
-            fmt += ' using ' + ':'.join( str(x+1) for x in range(using_Ncolumns) )
+            using = curve.get('using')
+            if using is None:
+                using = ':'.join(str(x+1) for x in range(using_Ncolumns))
+            fmt += ' using ' + using
 
             # to test the plot I plot a single record
             fmtTest = fmt
@@ -1382,22 +1478,24 @@ labels with spaces in them
         if self.plotOptions.get('3d'): basecmd += 'splot '
         else:                          basecmd += 'plot '
 
+        plotCurveCmdsNonData = []
+        plotCurveCmds        = []
+        plotCurveCmdsMinimal = [] # same as above, but with a single data point per plot only
+
         # send all equations
         if 'equation' in self.plotOptions:
             if isinstance(self.plotOptions['equation'], (list, tuple)):
-                basecmd += ''.join( eq + ', ' for eq in self.plotOptions['equation'])
+                plotCurveCmdsNonData += self.plotOptions['equation']
             else:
-                basecmd += self.plotOptions['equation'] + ', '
+                plotCurveCmdsNonData.append(self.plotOptions['equation'])
 
         if 'rgbimage' in self.plotOptions:
             if not os.access     (self.plotOptions['rgbimage'], os.R_OK) or \
                not os.path.isfile(self.plotOptions['rgbimage']):
                 raise GnuplotlibError("Requested image '{}' is not a readable file".format(self.plotOptions['rgbimage']))
 
-            basecmd += '"{0}" binary filetype=auto flipy with rgbimage title "{0}", '.format(self.plotOptions['rgbimage'])
+            plotCurveCmdsNonData.append('"{0}" binary filetype=auto flipy with rgbimage title "{0}"'.format(self.plotOptions['rgbimage']))
 
-        plotCurveCmds        = []
-        plotCurveCmdsMinimal = [] # same as above, but with a single data point per plot only
         testData             = '' # data to make a minimal plot
 
         for curve in curves:
@@ -1427,7 +1525,10 @@ labels with spaces in them
                 # for some things gnuplot has its own implicit-tuples logic; I want to
                 # suppress this, so I explicitly tell gnuplot to use all the columns we
                 # have
-                using = ' using ' + ':'.join(str(x+1) for x in range(curve['tuplesize']))
+                using = curve.get('using')
+                if using is None:
+                    using = ':'.join(str(x+1) for x in range(curve['tuplesize']))
+                using = ' using ' + using
 
                 # I'm using ascii to talk to gnuplot, so the minimal and "normal" plot
                 # commands are the same (point count is not in the plot command)
@@ -1450,8 +1551,9 @@ labels with spaces in them
                 testData += testData_curve
 
         # the command to make the plot and to test the plot
-        cmd        =  basecmd + ','.join(plotCurveCmds)
-        cmdMinimal = (basecmd + ','.join(plotCurveCmdsMinimal)) if plotCurveCmdsMinimal else cmd
+        cmd        =  basecmd + ','.join(plotCurveCmdsNonData + plotCurveCmds)
+        cmdMinimal = (basecmd + ','.join(plotCurveCmdsNonData + plotCurveCmdsMinimal)) \
+            if plotCurveCmdsMinimal else cmd
 
         return (cmd, cmdMinimal, testData)
 
@@ -1468,17 +1570,18 @@ labels with spaces in them
         # downstream can assume we have arrays
 
         # convert any scalars in the data list
-        curves = [ np.array((c,)) if isinstance(c, numbers.Real) else c for c in curves ]
-        if all(type(curve) is np.ndarray for curve in curves):
-            curves = (list(curves),)
-        elif all(type(curve) is tuple for curve in curves):
-            # we have a list of tuples. I convert this into a list of lists, and
-            # each scalar in each list becomes a numpy array
-            curves = [ [ np.array((c,)) if isinstance(c, numbers.Real) else c
-                         for c in curve ]
-                       for curve in curves ]
-        else:
-            raise GnuplotlibError("all data arguments should be of type ndarray (one curve) or tuples")
+        if len(curves):
+            curves = [ np.array((c,)) if isinstance(c, numbers.Real) else c for c in curves ]
+            if all(type(curve) is np.ndarray for curve in curves):
+                curves = (list(curves),)
+            elif all(type(curve) is tuple for curve in curves):
+                # we have a list of tuples. I convert this into a list of lists, and
+                # each scalar in each list becomes a numpy array
+                curves = [ [ np.array((c,)) if isinstance(c, numbers.Real) else c
+                             for c in curve ]
+                           for curve in curves ]
+            else:
+                raise GnuplotlibError("all data arguments should be of type ndarray (one curve) or tuples")
 
         # add an options dict if there isn't one, apply the base curve
         # options to each curve
@@ -1624,39 +1727,50 @@ labels with spaces in them
         # number of data points
         plotcmd, testcmd, testdata = self._getPlotCmd( curves )
 
-        self._testPlotcmd(testcmd, testdata)
+        if self.plotOptions.get('terminal') == 'gp':
+            self._dumpPipe = open(self.plotOptions['output'],'w')
+            os.chmod(self.plotOptions['output'], 0o755)
 
-        # tests ok. Now set the terminal and actually make the plot!
-        if 'terminal' in self.plotOptions:
-            self._safelyWriteToPipe("set terminal " + self.plotOptions['terminal'],
-                                    'terminal')
+            import distutils.spawn
+            gnuplotpath = distutils.spawn.find_executable('gnuplot')
+
+            self._safelyWriteToPipe('#!' + gnuplotpath)
+            self._safelyWriteToPipe(self._plotOptionsCmds)
 
-        # I always set the output. If no plot option explicitly is given then I
-        # either "set output" for a known interactive terminal, or redirect to
-        # python's STDOUT otherwise
-        if 'output' in self.plotOptions:
-            if self.plotOptions['output'] != '':
-                # user requested an explicit output
-                self._safelyWriteToPipe('set output "' + self.plotOptions['output'] + '"',
-                                        'output')
-            else:
-                # user requested null output
-                self._safelyWriteToPipe('set output',
-                                        'output')
         else:
-            # user requested nothing. Is this a known interactive terminal or an
-            # unspecified terminal (unspecified terminal assumed to be
-            # interactive)? Then set the null output
-            if 'terminal' not in self.plotOptions or self.plotOptions['terminal'] in knownInteractiveTerminals:
-                self._safelyWriteToPipe('set output',
-                                        'output')
-            else:
-                if self.fdDupSTDOUT is None:
-                    raise Exception("I need to plot to STDOUT, but STDOUT wasn't available")
+            self._testPlotcmd(testcmd, testdata)
 
-                self.plotOptions['output'] = '/dev/fd/' + str(self.fdDupSTDOUT)
-                self._safelyWriteToPipe('set output "' + self.plotOptions['output'] + '"',
-                                        'output')
+            # tests ok. Now set the terminal and actually make the plot!
+            if 'terminal' in self.plotOptions:
+                self._safelyWriteToPipe("set terminal " + self.plotOptions['terminal'],
+                                        'terminal')
+
+            # I always set the output. If no plot option explicitly is given then I
+            # either "set output" for a known interactive terminal, or redirect to
+            # python's STDOUT otherwise
+            if 'output' in self.plotOptions:
+                if self.plotOptions['output'] != '':
+                    # user requested an explicit output
+                    self._safelyWriteToPipe('set output "' + self.plotOptions['output'] + '"',
+                                            'output')
+                else:
+                    # user requested null output
+                    self._safelyWriteToPipe('set output',
+                                            'output')
+            else:
+                # user requested nothing. Is this a known interactive terminal or an
+                # unspecified terminal (unspecified terminal assumed to be
+                # interactive)? Then set the null output
+                if 'terminal' not in self.plotOptions or self.plotOptions['terminal'] in knownInteractiveTerminals:
+                    self._safelyWriteToPipe('set output',
+                                            'output')
+                else:
+                    if self.fdDupSTDOUT is None:
+                        raise Exception("I need to plot to STDOUT, but STDOUT wasn't available")
+
+                    self.plotOptions['output'] = '/dev/fd/' + str(self.fdDupSTDOUT)
+                    self._safelyWriteToPipe('set output "' + self.plotOptions['output'] + '"',
+                                            'output')
 
         # all done. make the plot
         self._printGnuplotPipe( plotcmd + "\n")
@@ -1664,13 +1778,23 @@ labels with spaces in them
         for curve in curves:
             self._sendCurve(curve)
 
-        # read and report any warnings that happened during the plot
-        self._checkpoint('printwarnings')
+        if self.plotOptions.get('terminal') == 'gp':
+            self._printGnuplotPipe('pause mouse close\n')
+            self._dumpPipe.close()
+            self._dumpPipe = None
 
-        # Reset the output. This is required for some terminals such as svg,
-        # that need to write out a closing stanza
-        self._safelyWriteToPipe('set output', 'output')
+        else:
+            # read and report any warnings that happened during the plot
+            self._checkpoint('printwarnings')
 
+            # Reset the output. This is required for some terminals such as svg,
+            # that need to write out a closing stanza
+            self._safelyWriteToPipe('set output', 'output')
+
+            if self.plotOptions.get('wait'):
+                self._printGnuplotPipe('pause mouse close\n')
+                self._logEvent("Waiting for data from gnuplot")
+                self._checkpoint('waitforever')
 
 
 
@@ -1778,7 +1902,7 @@ def plot(*curves, **jointOptions):
     # object. There's no gnuplot session to reuse in that case, and otherwise
     # the dumping won't get activated
     global globalplot
-    if not globalplot or plotOptions.get('dump'):
+    if not globalplot or _data_dump_only(plotOptions):
         globalplot = gnuplotlib(**plotOptions)
     else:
         globalplot.__init__(**plotOptions)
